function T = implicit2D_HTE_1333(nx,ny,dx,dy,ke,alpha,t,Tw,Tinf,Ts0)
% Copyright (C) 2023 - Manuel Regueiro-Picallo
% 
% Implictit Solver 2D heat transfer equation 
% dT/dt = ke * [d2T/dx2 + d2T/dy2]
%
% nx ----> number of elements in X-axis
% ny ----> number of elements in Y-axis
% dx ----> spatial resolution in X-axis (m)
% dy ----> spatial resolution in Y-axis (m)
% ke ----> thermal diffusivity (m^2/s)
% alpha -> leakage coefficient (m^-1)
% t  ----> time vector (s)
% Tw ----> boundary condition. Temperature timeseries (C)
% Tinf --> boundary condition. External Temperature timeseries (C)
% Ts0 ---> initial conditions. Constant temperature profile (C)
%
% Initial  conditions - Temperature profile
% Boundary conditions 
%   * Top: Dirichlet BC -----------> Tsup = f(t) = Twater(t)
%   * Bottom & walls: Cauchy BC ---> -kdT/dx = h(T-Tinf)
%
% Reference coordinate system (1333)
% 	1:   Top BC, type 1 (Dirichlet)
% 	333: Bottom & walls BC, type 3 (Cauchy)
%
% Function
%
% Author: Manuel Regueiro-Picallo <manuel.regueiro1@udc.es>

% Time resolution
t_total = t(end);       %total time, s
nt      = length(t);    %total number of time steps
dt      = t_total/nt;   %timestep, s

% Numerical parameters
Bi_x    = alpha*dx;       % Biot number, X-axis
Bi_y    = alpha*dy;       % Biot number, Y-axis
d_x     = ke*dt/dx^2;     % diffusion number, X-axis
d_y     = ke*dt/dy^2;     % diffusion number, Y-axis

% Stability of the numerical solution (only for implicit method)
d  = max([d_x d_y]);
r     = 1/(1+4*d);      % stability parameter
if r<=1
    %fprintf('solution stable (0 < r < 1)\nr = %8.4f\n',r)
else
    fprintf('solution unstable (r > 1)\nr = %8.4f\n',r)
end

% Solution AA*xy(n+1)=xy(n)+b. Create A matrix
% First A - matrix: A1. Wall
A1 = zeros(ny,ny);
for i=1:ny-1
    A1(i,i)   =  1+2*d_x+2*d_y+2*d_x*Bi_x; % BC left/right side (due to Cauchy BC)
    A1(i,i+1) = -d_y;             % upper diagonal
    A1(i+1,i) = -d_y;             % lower diagonal
end
    A1(1,1)     = 1+2*d_x+2*d_y+2*d_x*Bi_x; % BC bottom-left/right side (due to Cauchy BC)
    A1(1,2)     = -1*d_y;         % BC bottom-left/right side (due to Cauchy BC)
    A1(ny,ny-1) = -2*d_y;         % BC top-left/right    side (due to Cauchy BC)
    A1(ny,ny)   = 1+2*d_x+2*d_y+2*d_x*Bi_x+2*d_y*Bi_y; % BC top-left/right    side (due to Cauchy BC)

% Second A - matrix: A2. Wall
A2 = zeros(ny,ny);
for i=1:ny-1
    A2(i,i)   = 1+2*d_x+2*d_y;    % diagonal
    A2(i,i+1) = -d_y;             % upper diagonal
    A2(i+1,i) = -d_y;             % lower diagonal
end
    A2(ny,ny-1) = -2*d_y;         % BC top-left/right    side (due to Cauchy BC)
    A2(ny,ny)   = 1+2*d_x+2*d_y+2*d_y*Bi_y; % BC top-left/right    side (due to Cauchy BC)

% Identity matrix: I1
I1 =   -d_x*eye(ny);

% Identity matrix: I2
I2 = -2*d_x*eye(ny);    

% Build AA matrix
m  = nx*ny; % number of elements
AA = zeros(m,m);
for i=2:nx-1
    aux = (i-1)*ny;
    AA(aux+1:aux+ny,aux+1:aux+ny) = A2;      % diagonal
    AA(aux+1:aux+ny,aux+ny+1:aux+2*ny) = I1; % upper diagonal
    AA(aux+1:aux+ny,aux-ny+1:aux) = I1;      % lower diagonal
end
    AA(1:ny,1:ny)              = A1; 
    AA(1:ny,ny+1:2*ny)         = I2; 
    AA(m-ny+1:m,m-2*ny+1:m-ny) = I2; 
    AA(m-ny+1:m,m-ny+1:m)      = A1; 

    [L,U,P] = lu(AA);
% Allocating Temperatures and setting initial conditions
T      = zeros(m,nt);  % solution array
B      = zeros(m,1);   % BC vector
T(:,1) = Ts0;          % initial conditions. Uniform Temp. Vector profile
Tsup   = Tw;
Ti     = Tinf.*ones(1,nt);

% Implicit Sover
for k=2:nt
    B(1)          = 2*d_x*Bi_x*Ti(k)+d_y*Tsup(k);       % Cauchy    BC
    B(2:ny-1)     = 2*d_x*Bi_x*Ti(k);                   % Cauchy    BC
    B(ny)         = 2*d_x*Bi_x*Ti(k)+2*d_y*Bi_y*Ti(k);  % Cauchy    BC
    for j=2:nx-1
        aux       = (j-1)*ny;
        B(aux+1)  = d_y*Tsup(k);                        % Dirichlet BC
        B(j*ny)   = 2*d_y*Bi_y*Ti(k);                   % Cauchy    BC
    end
    B(m-ny+1)     = 2*d_x*Bi_x*Ti(k)+d_y*Tsup(k);       % Cauchy    BC
    B(m-ny+2:m-1) = 2*d_x*Bi_x*Ti(k);                   % Cauchy    BC
    B(m)          = 2*d_x*Bi_x*Ti(k)+2*d_y*Bi_y*Ti(k);  % Cauchy    BC
    
    y = L\(P*(B+T(:,k-1)));
    T(:,k) = U\y;    
    %T(:,k)= AA\(B+T(:,k-1));
end  

T = reshape(T,[ny,nx,nt]);
end